import gdsfactory as gf

from ubc.tech import LAYER
from ubc.add_pins import add_pins


@gf.cell
def straight(
    length: float = 10.0,
    width: float = 0.5,
    layer: gf.types.Layer = LAYER.WG,
    with_pins: bool = True,
    **kwargs,
) -> gf.Component:
    """Straight waveguide.

    Args:
        length:
        width:
        layer:
        with_pins:
    """
    c = gf.Component()

    s = gf.components.straight(length=length, width=width, layer=layer, **kwargs)
    ref = c << s
    c.add_ports(ref.ports)
    c.copy_child_info(s)

    if with_pins:
        labels = [
            "Lumerical_INTERCONNECT_library=Design kits/EBeam",
            "Lumerical_INTERCONNECT_component=ebeam_wg_integral_1550",
            f"Spice_param:wg_width={width:.3f}u wg_length={length:.3f}u",
        ]

        for i, text in enumerate(labels):
            c.add_label(text=text, position=(length / 2, i * 0.1), layer=LAYER.DEVREC)
        add_pins(c)
    return c


if __name__ == "__main__":
    c = straight()
    c.show()



import numpy as np
import pytest

from gdslib.simphony.circuit import get_transmission

from ubcsp import circuit_factory
from ubcsp import component_factory
from ubcsp import circuit_names
from ubcsp import component_names


@pytest.mark.parametrize("component_type", component_names)
def test_elements(component_type, data_regression):
    c = component_factory[component_type]()
    wav = np.linspace(1520, 1570, 3) * 1e-9
    f = 3e8 / wav
    s = c.s_parameters(freq=f)
    _, rows, cols = np.shape(s)
    sdict = {
        f"S{i+1}{j+1}": np.round(np.abs(s[:, i, j]), decimals=3).tolist()
        for i in range(rows)
        for j in range(cols)
    }
    data_regression.check(sdict)


@pytest.mark.parametrize("component_type", circuit_names)
def test_circuits(component_type, data_regression):
    c = circuit_factory[component_type]()
    r = get_transmission(c, num=3)
    s = np.round(r["s"], decimals=3).tolist()
    data_regression.check(dict(w=r["wavelength_nm"].tolist(), s=s))



from typing import Callable

from gdsfactory.add_pins import add_pin_square
from gdsfactory.component import Component
from gdsfactory.types import Layer
from ubc.tech import LAYER


def add_pins(
    component: Component,
    function: Callable = add_pin_square,
    layer_port: Layer = LAYER.WG,
    layer_pin: Layer = LAYER.PORT,
    layer_label: Layer = LAYER.LABEL,
) -> None:

    for p in component.get_ports_list(layer=layer_port):
        function(component=component, port=p, layer=layer_pin, layer_label=layer_label)


if __name__ == "__main__":
    import gdsfactory as gf

    c = gf.c.straight()
    add_pins(c)
    c.show(show_ports=False)



import gdslib as gl
from gdslib import autoname
from gdslib.types import ModelFactory
import gdsfactory as gf
from simphony.library import siepic
from simphony.netlist import Subcircuit


@autoname
def mzi(
    length_y: float = 1.0,
    delta_length: float = 100.0,
    length_x: float = 10.0,
    y_model_factory: ModelFactory = siepic.ebeam_y_1550,
    waveguide: ModelFactory = siepic.ebeam_wg_integral_1550,
) -> Subcircuit:
    """Mzi circuit model

    Args:
        length_y: vertical length for both and top arms (um)
        delta_length: bottom arm extra length
        length_x: horizontal length for both and top arms (um)
        waveguide: waveguide_model

    .. code::

                   __Lx__
                  |      |
                  Ly     Lyr (not a parameter)
                  |      |
        splitter==|      |==combiner
                  |      |
                  Ly     Lyr (not a parameter)
                  |      |
                  | delta_length/2
                  |      |
                  |__Lx__|



    .. plot::
      :include-source:

      import gdsfactory as gf

      c = gf.c.mzi(length_y=0.1, delta_length=0, length_x=10)
      gf.plotgds(c)


    .. plot::
        :include-source:

        import ubc
        import gdslib as gl

        c = ubc.circuits.mzi()
        gl.plot_circuit(c)

    """
    y = gf.call_if_func(y_model_factory)
    wg_long = waveguide(length=(2 * length_y + delta_length + length_x) * 1e-6)
    wg_short = waveguide(length=(2 * length_y + length_x) * 1e-6)

    # Create the circuit, add all individual instances
    circuit = Subcircuit("mzi")
    circuit.add(
        [
            (y, "splitter"),
            (y, "recombiner"),
            (wg_long, "wg_long"),
            (wg_short, "wg_short"),
        ]
    )
    circuit.elements["splitter"].pins = ("in1", "out1", "out2")
    circuit.elements["recombiner"].pins = ("out1", "in2", "in1")

    # Circuits can be connected using the elements' string names:
    circuit.connect_many(
        [
            ("splitter", "out1", "wg_long", "n1"),
            ("splitter", "out2", "wg_short", "n1"),
            ("recombiner", "in1", "wg_long", "n2"),
            ("recombiner", "in2", "wg_short", "n2"),
        ]
    )
    circuit.elements["splitter"].pins["in1"] = "input"
    circuit.elements["recombiner"].pins["out1"] = "output"
    return circuit


if __name__ == "__main__":
    import matplotlib.pyplot as plt

    m = mzi()
    gl.simphony.plot_circuit(m)
    plt.show()



import gdsfactory as gf
from gdslib.simphony import plot_circuit
from simphony.library import siepic
from simphony.netlist import Subcircuit


def add_grating_coupler(circuit, grating_coupler=siepic.ebeam_gc_te1550):
    """Add input and output gratings couplers.

    Args:
        circuit: needs to have `input` and `output` pins
        gc: grating coupler model function
    """
    c = Subcircuit(f"{circuit}_gc")
    gc = gf.call_if_func(grating_coupler)
    c.add([(gc, "gci"), (gc, "gco"), (circuit, "circuit")])
    c.connect_many(
        [("gci", "n1", "circuit", "input"), ("gco", "n1", "circuit", "output")]
    )

    # c.elements["circuit"].pins["input"] = "input_circuit"
    # c.elements["circuit"].pins["output"] = "output_circuit"
    c.elements["gci"].pins["n2"] = "input"
    c.elements["gco"].pins["n2"] = "output"

    return c


if __name__ == "__main__":
    import matplotlib.pyplot as plt
    from mzi import mzi

    c1 = mzi()
    c2 = add_grating_coupler(c1)
    plot_circuit(c2)
    plt.show()



from typing import Tuple
from omegaconf import OmegaConf
from pathlib import Path
import gdsfactory as gf

import ubc
from ubc.tech import LAYER
from ubc.config import PATH

floorplan_size = (605, 410)
add_gc = ubc.components.add_fiber_array


def write_mask_gds_with_metadata(m) -> Tuple[Path, Path]:
    """Returns"""
    gdspath = PATH.mask / f"{m.name}.gds"
    m.write_gds_with_metadata(gdspath=gdspath)

    labels_path = gf.mask.write_labels(gdspath=gdspath, layer_label=LAYER.LABEL)
    metadata_path = gdspath.with_suffix(".yml")
    test_metadata_path = gdspath.with_suffix(".tp.yml")
    mask_metadata = OmegaConf.load(metadata_path)

    tm = gf.mask.merge_test_metadata(
        labels_path=labels_path, mask_metadata=mask_metadata
    )
    test_metadata_path.write_text(OmegaConf.to_yaml(tm))
    return m, tm


def test_mask2():
    """spirals for extracting straight waveguide loss"""
    N = 15
    radius = 15

    e = []
    e.append(
        ubc.components.add_fiber_array(
            component=ubc.components.spiral(
                N=N,
                radius=radius,
                y_straight_inner_top=0,
                x_inner_length_cutback=0,
                info=dict(does=["spiral", "te1550"]),
            )
        )
    )
    e.append(
        ubc.components.add_fiber_array(
            component=ubc.components.spiral(
                N=N,
                radius=radius,
                y_straight_inner_top=30,
                x_inner_length_cutback=85,
            )
        )
    )

    c = gf.pack(e)

    m = c[0]
    m.name = "EBeam_JoaquinMatres_2"
    m << gf.c.rectangle(size=floorplan_size, layer=LAYER.FLOORPLAN)
    return write_mask_gds_with_metadata(m)


def test_mask1():
    e = [add_gc(ubc.components.straight())]
    e += [add_gc(gf.components.mzi(delta_length=dl)) for dl in [9.32, 93.19]]
    e += [
        add_gc(gf.components.ring_single(radius=12, gap=gap, length_x=coupling_length))
        for gap in [0.2]
        for coupling_length in [2.5, 4.5, 6.5]
    ]

    e += [
        add_gc(ubc.components.dbr_cavity(w0=w0, dw=dw))
        for w0 in [0.5]
        for dw in [50e-3, 100e-3, 150e-3, 200e-3]
    ]
    e += [add_gc(ubc.components.ring_with_crossing())]
    e += [add_gc(ubc.components.ring_with_crossing(with_component=False))]

    c = gf.pack(e, max_size=floorplan_size)
    m = c[0]
    m.name = "EBeam_JoaquinMatres_1"
    m << gf.c.rectangle(size=floorplan_size, layer=LAYER.FLOORPLAN)
    return write_mask_gds_with_metadata(m)


def test_mask3():
    """contains mirror cavities and structures inside a resonator"""
    e = [add_gc(ubc.components.crossing())]
    e += [add_gc(ubc.components.dc_adiabatic(), optical_routing_type=1)]
    e += [add_gc(ubc.components.dc_broadband_te())]
    e += [add_gc(ubc.components.y_splitter(), optical_routing_type=1)]
    e += [add_gc(ubc.components.y_adiabatic(), optical_routing_type=1)]
    c = gf.pack(e)
    m = c[0]
    m.name = "EBeam_JoaquinMatres_3"
    m << gf.c.rectangle(size=floorplan_size, layer=LAYER.FLOORPLAN)
    return write_mask_gds_with_metadata(m)


if __name__ == "__main__":
    m, tm = test_mask1()
    m, tm = test_mask2()
    m, tm = test_mask3()
    m.show()



def chop(x, y, ymax=None, ymin=None, xmin=None, xmax=None):
    """Chops x, y."""

    if xmax:
        y = y[x < xmax]
        x = x[x < xmax]

    if xmin:
        y = y[x > xmin]
        x = x[x > xmin]

    if ymax:
        x = x[y < ymax]
        y = y[y < ymax]

    if ymin:
        x = x[y > ymin]
        y = y[y > ymin]

    return x, y



from .add_grating_coupler import add_grating_coupler
from .mzi import mzi
from .mzi_spectrum import mzi_spectrum
import gdslib

circuit_factory = dict(mzi=mzi)
component_factory = dict()

circuit_names = list(circuit_factory.keys())
component_names = list(component_factory.keys())


__all__ = ["add_grating_coupler", "mzi", "mzi_spectrum"]


if gdslib.__version__ < "0.3.0":
    raise ValueError(
        f"gdslib version = {gdslib.__version__}"
        " you need to upgrade gdslib `pip install gdslib`"
    )



""" data analysis module
"""


from .chop import chop
from .find_bandwidth import find_bandwidth
from .read_mat import read_mat
from .remove_baseline import remove_baseline
from .windowed_mean import windowed_mean

__all__ = ["read_mat", "remove_baseline", "chop", "windowed_mean", "find_bandwidth"]



from scipy.io import loadmat


def read_mat(filename, port=0):
    """reads .mat file and returns np.array
    input: (.mat data download filename, port response)
    outputs parsed data array [wavelength (m), power (dBm)]
    data is assumed to be from automated measurement scanResults or scandata format

    based on SiEPIC_Photonics_Package/core.py
    """
    data = loadmat(filename)

    if "scanResults" in data:
        wavelength = data["scanResults"][0][port][0][:, 0]
        power = data["scanResults"][0][port][0][:, 1]
    elif "scandata" in data:
        wavelength = data["scandata"][0][0][0][:][0]
        power = data["scandata"][0][0][1][:, port]
    elif "wavelength" in data:
        wavelength = data["wavelength"][0][:]
        power = data["power"][:, port][:]

    return [wavelength, power]


if __name__ == "__main__":
    import matplotlib.pyplot as plt
    import ubc

    w, p = read_mat(ubc.PATH.mzi1)
    plt.plot(w, p)
    plt.show()



"""UBC Siepic Ebeam PDK from edx course"""

import gdsfactory as gf
from ubc.config import CONFIG, PATH
from ubc.tech import LAYER, strip

from ubc import da
from ubc import components
from ubc import tech


lys = gf.layers.load_lyp(PATH.lyp)


__all__ = [
    "CONFIG",
    "da",
    "PATH",
    "components",
    "tech",
    "strip",
    "LAYER",
]
__version__ = "0.0.10"



""" wavelength dependent effective index model
based on https://github.com/SiEPIC-Kits/SiEPIC_Photonics_Package
"""

import numpy as np

wavelength_start = 1500e-9
wavelength_stop = 1600e-9
resolution = 0.001
wavelength_um = (
    np.linspace(
        wavelength_start,
        wavelength_stop,
        round((wavelength_stop - wavelength_start) * 1e9 / resolution),
    )
    * 1e6
)


def neff(wavelength_um=wavelength_um, n1=2.4, n2=-1.0, n3=0.0, wavelength0_um=1.55):
    """waveguide model neff"""
    w = wavelength_um
    w0 = wavelength0_um
    return n1 + n2 * (w - w0) + n3 * (w - w0) ** 2


def beta(wavelength_um=wavelength_um, alpha=1e-3, neff=neff, n1=2.4, n2=-1, n3=0):
    """propagation constant

    Args:
        wavelength_um (m):
        alpha: propagation loss [micron^-1] constant
    """
    if callable(neff):
        neff = neff(wavelength_um, n1=n1, n2=n2, n3=n3)

    return 2 * np.pi * neff / wavelength_um - 1j * alpha / 2 * np.ones(
        np.size(wavelength_um)
    )


if __name__ == "__main__":
    import matplotlib.pyplot as plt

    plt.plot(wavelength_um, neff(wavelength_um))
    plt.show()



import pathlib

import gdsfactory as gf
from numpy import ndarray
from gdsfactory.component import Component
from gdsfactory.port import auto_rename_ports
from ubc.tech import LAYER

cwd = pathlib.Path(__file__).parent.absolute()
gds = cwd / "gds"


layer = LAYER.WG
port_width = 0.5


def guess_port_orientaton(position: ndarray, name: str, label: str, n: int) -> int:
    """we assume that ports with x<0 are inputs (orientation=180deg)
    and ports with x>0 are outputs
    """
    p = position
    if "gc" in name:
        return 0
    if label == "opt1":
        return 180
    if p[1] > 0 and "crossing" in name:
        return 90
    if p[1] < 0 and "crossing" in name:
        return 270
    if n == 4 and label in ["opt1", "opt2"]:
        return 180
    if n == 4 and label in ["opt3", "opt4"]:
        return 0
    if p[0] <= 0:
        return 180
    return 0


@gf.functions.cache
def import_gds(gdsname: str, rename_ports: bool = False) -> Component:
    """import gds from SIEPIC PDK"""
    c = gf.import_gds(gds / f"{gdsname}.gds")
    c.function_name = gdsname

    n = 0
    for label in c.get_labels():
        if label.text.startswith("opt"):
            n += 1

    for label in c.get_labels():
        if label.text.startswith("opt"):
            port_name = label.text
            port = gf.Port(
                name=port_name,
                midpoint=label.position,
                width=port_width,
                orientation=guess_port_orientaton(
                    position=label.position,
                    name=gdsname,
                    label=label.text,
                    n=n,
                ),
                layer=layer,
            )
            if port_name not in c.ports:
                c.add_port(port)

    if rename_ports:
        auto_rename_ports(c)

    return c


if __name__ == "__main__":
    gdsname = "ebeam_y_1550"
    c = import_gds(gdsname)
    print(c.ports)
    c.show()



"""Technology definitions

- LayerStack
- cross_sections (xs_)
- constants (WIDTH, CLADDING_OFFSET ...)
- adapted gdsfactory.components to this technology
"""

import pydantic

import gdsfactory as gf
from gdsfactory.tech import LayerStack, LayerLevel
from gdsfactory.types import Layer
import gdsfactory.simulation as sim
import gdsfactory.simulation.lumerical as lumerical

from ubc.config import PATH

nm = 1e-3


@pydantic.dataclasses.dataclass(frozen=True)
class LayerMapUbc:
    WG: Layer = (1, 0)
    WG2: Layer = (31, 0)
    DEVREC: Layer = (68, 0)
    LABEL: Layer = (10, 0)
    PORT: Layer = (1, 10)
    FLOORPLAN: Layer = (99, 0)


LAYER = LayerMapUbc()


def get_layer_stack_ubc(thickness: float = 220 * nm) -> LayerStack:
    """Returns generic LayerStack"""
    return LayerStack(
        strip=LayerLevel(
            layer=LAYER.WG,
            thickness=thickness,
            zmin=0.0,
            material="si",
        ),
        strip2=LayerLevel(
            layer=LAYER.WG2,
            thickness=thickness,
            zmin=0.0,
            material="si",
        ),
    )


LAYER_STACK = get_layer_stack_ubc()
layer_set = gf.layers.load_lyp(PATH.lyp)
to_3d = gf.partial(
    gf.export.to_3d,
    layer_set=layer_set,
    layer_stack=LAYER_STACK,
)

write_sparameters_lumerical = gf.partial(
    lumerical.write_sparameters_lumerical,
    layer_stack=LAYER_STACK,
    dirpath=PATH.sparameters,
)

get_sparameters_data_lumerical = gf.partial(
    sim.get_sparameters_data_lumerical,
    layer_stack=LAYER_STACK,
    dirpath=PATH.sparameters,
)


strip = gf.partial(gf.cross_section.strip, layers_cladding=(LAYER.DEVREC,))



import numpy as np
import scipy.signal as sig


def windowed_mean(data: np.array, n: int = 60) -> np.array:
    """from a data1d array returns the smoothen data using a window averaging

    Args:
        data: data array
        n: points per window

     Returns:
       smoothened data array
    """

    dims = len(data.shape)
    s = sig.convolve(data, np.ones((2 * n + 1,) * dims), mode="same")
    d = sig.convolve(np.ones_like(data), np.ones((2 * n + 1,) * dims), mode="same")
    return s / d


if __name__ == "__main__":
    from ubc.da.read_mat import read_mat
    from ubc.config import PATH
    import matplotlib.pyplot as plt

    wavelength, power = read_mat(PATH.ring_te_r3_g100)
    power_envelope = windowed_mean(power, 60)
    plt.plot(wavelength, power, label="power")
    plt.plot(wavelength, power_envelope, label="envelope")
    plt.legend()
    plt.show()



"""Loads a default_config from this file.
Overwrites config with a `config.yml` in the current working directory (if any).
"""

__all__ = ["PATH", "CONFIG"]

import io
import pathlib

from omegaconf import OmegaConf

default_config = io.StringIO(
    """
username: JoaquinMatres
"""
)


cwd = pathlib.Path.cwd()
cwd_config = cwd / "config.yml"
config_base = OmegaConf.load(default_config)
module_path = pathlib.Path(__file__).parent.absolute()
repo_path = module_path.parent

try:
    config_cwd = OmegaConf.load(cwd_config)
except Exception:
    config_cwd = OmegaConf.create()
CONFIG = OmegaConf.merge(config_base, config_cwd)


class Path:
    module = module_path
    repo = repo_path
    data = repo_path / "data"
    sparameters = module / "sparameters"
    mask = module_path / "mask"

    mzi = data / "mzi"
    mzi1 = mzi / "ZiheGao_MZI1_272_Scan1.mat"
    mzi3 = mzi / "ZiheGao_MZI2_271_Scan1.mat"
    mzi1 = mzi / "ZiheGao_MZI3_270_Scan1.mat"
    mzi4 = mzi / "ZiheGao_MZI4_269_Scan1.mat"
    mzi5 = mzi / "ZiheGao_MZI5_268_Scan1.mat"
    mzi6 = mzi / "ZiheGao_MZI6_267_Scan1.mat"
    mzi8 = mzi / "ZiheGao_MZI8_266_Scan1.mat"
    mzi11 = mzi / "ZiheGao_MZI1_273_Scan1.mat"
    mzi17 = mzi / "ZiheGao_MZI17_265_Scan1.mat"
    ring = data / "ring"
    ring_te_r3_g100 = ring / "LukasC_RingDoubleTER3g100_1498.mat"
    ring_te_r3_g150 = ring / "LukasC_RingDoubleTER3g150_1497.mat"
    ring_te_r10_g50 = ring / "LukasC_RingDoubleTER10g50_1496.mat"
    ring_te_r10_g100 = ring / "LukasC_RingDoubleTER10g100_1495.mat"
    ring_te_r10_g150 = ring / "LukasC_RingDoubleTER10g150_1494.mat"
    ring_te_r10_g200 = ring / "LukasC_RingDoubleTER10g200_1493.mat"
    ring_tm_r30_g150 = ring / "LukasC_RingDoubleTMR30g150_1492.mat"
    ring_tm_r30_g200 = ring / "LukasC_RingDoubleTMR30g200_1491.mat"
    ring_tm_r30_g250 = ring / "LukasC_RingDoubleTMR30g250_1490.mat"
    dbr = data / "bragg"
    lyp = module / "klayout" / "tech" / "layers.lyp"


PATH = Path()


if __name__ == "__main__":
    print(CONFIG.tech.layers_cladding.clad)
    print(PATH.data)



import gdsfactory as gf
from ubc.import_gds import import_gds


@gf.cell
def gc_te1550() -> gf.Component:
    c = import_gds("ebeam_gc_te1550")
    c = c.rotate(angle=180)
    c.info.polarization = "te"
    c.info.wavelength = 1.550
    c.auto_rename_ports()
    return c


@gf.cell
def gc_te1550_broadband() -> gf.Component:
    c = import_gds("ebeam_gc_te1550_broadband")
    c = c.rotate(angle=180)
    c.info.polarization = "te"
    c.info.wavelength = 1.550
    c.auto_rename_ports()
    gf.port.auto_rename_ports(c)
    return c


@gf.cell
def gc_te1310() -> gf.Component:
    c = import_gds("ebeam_gc_te1310")
    c = c.rotate(angle=180)
    c.info.polarization = "te"
    c.info.wavelength = 1.310
    c.auto_rename_ports()
    return c


@gf.cell
def gc_tm1550() -> gf.Component:
    c = import_gds("ebeam_gc_tm1550")
    c = c.rotate(angle=180)
    c.info.polarization = "tm"
    c.info.wavelength = 1.550
    c.auto_rename_ports()
    return c


if __name__ == "__main__":
    c = gc_te1550()
    c.show()



import pytest
from gdsfactory.component import Component
from gdsfactory.difftest import difftest
from pytest_regressions.data_regression import DataRegressionFixture
from ubc.components import factory


component_names = factory.keys()


@pytest.fixture(params=component_names, scope="function")
def component(request) -> Component:
    return factory[request.param]()


def test_pdk_gds(component: Component) -> None:
    """Avoid regressions in GDS geometry shapes and layers."""
    difftest(component)


def test_pdk_settings(
    component: Component, data_regression: DataRegressionFixture
) -> None:
    """Avoid regressions when exporting settings."""
    data_regression.check(component.to_dict())


def test_assert_ports_on_grid(component: Component):
    component.assert_ports_on_grid()



import matplotlib.pyplot as plt
import gdsfactory as gf
from scipy.constants import speed_of_light
from simphony.library import siepic
from simphony.netlist import Subcircuit
from simphony.simulation import MonteCarloSweepSimulation, SweepSimulation


def mzi_gc(
    L0=1,
    L1=100,
    L2=10,
    gc=siepic.ebeam_gc_te1550,
    y=siepic.ebeam_y_1550,
    wg=siepic.ebeam_wg_integral_1550,
):
    """Mzi with grating couplers

    Args:
        L0: vertical length for both and top arms
        L1: bottom arm extra length
        L2: L_top horizontal length

    .. code::

               __L2__
               |      |
               L0     L0r
               |      |
     splitter==|      |==recombiner
               |      |
               L0     L0r
               |      |
               L1     L1
               |      |
               |__L2__|


    .. plot::
      :include-source:

      import gdsfactory as gf

      c = gf.c.mzi(L0=0.1, L1=0, L2=10)
      gf.plotgds(c)

    """
    gc = gf.call_if_func(gc)
    y = gf.call_if_func(y)
    wg_long = wg(length=(2 * L0 + +L1 + L2) * 1e-6)
    wg_short = wg(length=(2 * L0 + L2) * 1e-6)

    # Create the circuit, add all individual instances
    circuit = Subcircuit("MZI")
    circuit.add(
        [
            (gc, "input"),
            (gc, "output"),
            (y, "splitter"),
            (y, "recombiner"),
            (wg_long, "wg_long"),
            (wg_short, "wg_short"),
        ]
    )

    # You can set pin names individually:
    circuit.elements["input"].pins["n2"] = "input"
    circuit.elements["output"].pins["n2"] = "output"

    # Or you can rename all the pins simultaneously:
    circuit.elements["splitter"].pins = ("in1", "out1", "out2")
    circuit.elements["recombiner"].pins = ("out1", "in2", "in1")

    # Circuits can be connected using the elements' string names:
    circuit.connect_many(
        [
            ("input", "n1", "splitter", "in1"),
            ("splitter", "out1", "wg_long", "n1"),
            ("splitter", "out2", "wg_short", "n1"),
            ("recombiner", "in1", "wg_long", "n2"),
            ("recombiner", "in2", "wg_short", "n2"),
            ("output", "n1", "recombiner", "out1"),
        ]
    )
    return circuit


def mzi_simulation(circuit=mzi_gc, **kwargs):
    """ Run a simulation on the netlist """
    c = circuit(**kwargs)

    simulation = SweepSimulation(c, 1500e-9, 1600e-9)
    result = simulation.simulate()

    f, s = result.data("input", "output")
    w = speed_of_light / f
    plt.plot(w * 1e9, s)
    plt.title("MZI")
    plt.xlabel("wavelength (nm)")
    plt.tight_layout()
    plt.show()


def mzi_variation(circuit=mzi_gc, **kwargs):
    """ Run a montercarlo simulation on the netlist """
    c = circuit(**kwargs)

    simulation = MonteCarloSweepSimulation(c, 1500e-9, 1600e-9)
    runs = 10
    result = simulation.simulate(runs=runs)
    for i in range(1, runs + 1):
        f, s = result.data("input", "output", i)
        plt.plot(f, s)

    # The data located at the 0 position is the ideal values.
    f, s = result.data("input", "output", 0)
    plt.plot(f, s, "k")
    plt.title("MZI Monte Carlo")
    plt.tight_layout()
    plt.show()


if __name__ == "__main__":
    mzi_simulation()



import gdsfactory as gf
from ubc.tech import strip


L = 1.55 / 4 / 2 / 2.44


def straight(cross_section: gf.types.CrossSectionFactory = strip, **kwargs):
    return gf.c.straight(cross_section=cross_section, **kwargs)


@gf.cell
def dbr(
    w0: float = 0.5, dw: float = 0.1, n: int = 600, l1: float = L, l2: float = L
) -> gf.Component:
    return gf.components.dbr(
        w1=w0 - dw / 2,
        w2=w0 + dw / 2,
        n=n,
        l1=l1,
        l2=l2,
        straight=straight,
    )


def dbr_cavity(**kwargs) -> gf.Component:
    return gf.c.cavity(component=dbr(**kwargs))


if __name__ == "__main__":
    c = dbr_cavity()
    c.show()



import gdsfactory as gf
from gdsfactory.component import Component
from ubc.import_gds import import_gds

from ubc.components.add_fiber_array import add_fiber_array
from ubc.components.grating_couplers import (
    gc_te1550,
    gc_te1550_broadband,
    gc_te1310,
    gc_tm1550,
)
from ubc.components.straight import straight
from ubc.components.dbr import dbr_cavity, dbr
from ubc.components.crossing import crossing, ring_with_crossing


def dc_broadband_te() -> Component:
    """Broadband directional coupler TE1550 50/50 power."""
    return import_gds("ebeam_bdc_te1550")


def dc_broadband_tm() -> Component:
    """Broadband directional coupler TM1550 50/50 power."""
    return import_gds("ebeam_bdc_tm1550")


def dc_adiabatic() -> Component:
    """Adiabatic directional coupler TE1550 50/50 power."""
    return import_gds("ebeam_adiabatic_te1550")


def y_adiabatic() -> Component:
    """Adiabatic Y junction TE1550 50/50 power."""
    return import_gds("ebeam_y_adiabatic")


def y_splitter() -> Component:
    """Y junction TE1550 50/50 power."""
    return import_gds("ebeam_y_1550")


spiral = gf.partial(gf.components.spiral_external_io)


factory = dict(
    add_fiber_array=add_fiber_array,
    crossing=crossing,
    dbr=dbr,
    dbr_cavity=dbr_cavity,
    dc_adiabatic=dc_adiabatic,
    dc_broadband_te=dc_broadband_te,
    dc_broadband_tm=dc_broadband_tm,
    gc_te1310=gc_te1310,
    gc_te1550=gc_te1550,
    gc_te1550_broadband=gc_te1550_broadband,
    gc_tm1550=gc_tm1550,
    ring_with_crossing=ring_with_crossing,
    spiral=spiral,
    straight=straight,
    y_adiabatic=y_adiabatic,
    y_splitter=y_splitter,
)


__all__ = list(factory.keys())


if __name__ == "__main__":
    # c = straight_no_pins()
    # c = add_fiber_array(component=c)
    # c = gc_tm1550()
    # print(c.get_ports_array())
    # print(c.ports.keys())
    c = straight()
    # c = add_fiber_array(component=c)
    c.show()



import matplotlib.pyplot as plt
import numpy as np


def find_bandwidth(x, y, threshold=3):
    """Finds the x bandwidth above a threshold around max(y)

    Args:
        x:
        y:
        threshold: default bandwidth point (in y scale units):
            3 for dB, 0.5 if Y is in linear scale from 0 to 1
    """
    index_max = np.argmax(y)
    ymax = y[index_max]
    ybw = ymax - threshold
    yl = y[:index_max]
    yr = y[index_max:]
    xl = x[:index_max]
    xr = x[index_max:]
    il = max(np.argwhere(np.abs(yl - ybw) < 0.1))
    ir = min(np.argwhere(np.abs(yr - ybw) < 0.1))
    return float(xr[ir] - xl[il])


def plot_bandwidth(x, y, threshold=3):
    index_max = np.argmax(y)
    ymax = y[index_max]
    ybw = ymax - threshold
    yl = y[:index_max]
    yr = y[index_max:]
    xl = x[:index_max]
    xr = x[index_max:]
    il = max(np.argwhere(np.abs(yl - ybw) < 0.1))
    ir = min(np.argwhere(np.abs(yr - ybw) < 0.1))

    plt.plot(x, y)
    plt.plot(xl[il], yl[il], "o", c="k")
    plt.plot(xr[ir], yr[ir], "o", c="k")
    plt.show()


if __name__ == "__main__":
    from ubc.da.dbr import dbrs
    from ubc.da.chop import chop
    from ubc.da.read_mat import read_mat

    w, p = read_mat(dbrs["1_5"], port=1)
    wc, pc = chop(w, p, ymin=-60)

    print(find_bandwidth(wc, pc) * 1e9)
    # plot_bandwidth(wc, pc)



from ubc.config import PATH

dbrs = {
    filename.split("_")[3][8:].replace("Num", "_"): PATH.dbr / filename
    for filename in [
        "ELEC_413_lukasc_BraggSet1Num10_1272.mat",
        "ELEC_413_lukasc_BraggSet1Num11_1273.mat",
        "ELEC_413_lukasc_BraggSet1Num12_1271.mat",
        "ELEC_413_lukasc_BraggSet1Num13_1278.mat",
        "ELEC_413_lukasc_BraggSet1Num14_1276.mat",
        "ELEC_413_lukasc_BraggSet1Num15_1277.mat",
        "ELEC_413_lukasc_BraggSet1Num16_1275.mat",
        "ELEC_413_lukasc_BraggSet1Num17_1282.mat",
        "ELEC_413_lukasc_BraggSet1Num18_1280.mat",
        "ELEC_413_lukasc_BraggSet1Num19_1281.mat",
        "ELEC_413_lukasc_BraggSet1Num1_1266.mat",
        "ELEC_413_lukasc_BraggSet1Num20_1279.mat",
        "ELEC_413_lukasc_BraggSet1Num21_1286.mat",
        "ELEC_413_lukasc_BraggSet1Num22_1284.mat",
        "ELEC_413_lukasc_BraggSet1Num23_1285.mat",
        "ELEC_413_lukasc_BraggSet1Num24_1283.mat",
        "ELEC_413_lukasc_BraggSet1Num2_1264.mat",
        "ELEC_413_lukasc_BraggSet1Num3_1265.mat",
        "ELEC_413_lukasc_BraggSet1Num4_1263.mat",
        "ELEC_413_lukasc_BraggSet1Num5_1270.mat",
        "ELEC_413_lukasc_BraggSet1Num6_1268.mat",
        "ELEC_413_lukasc_BraggSet1Num7_1269.mat",
        "ELEC_413_lukasc_BraggSet1Num8_1267.mat",
        "ELEC_413_lukasc_BraggSet1Num9_1274.mat",
        "ELEC_413_lukasc_BraggSet2Num10_1248.mat",
        "ELEC_413_lukasc_BraggSet2Num11_1249.mat",
        "ELEC_413_lukasc_BraggSet2Num12_1247.mat",
        "ELEC_413_lukasc_BraggSet2Num13_1254.mat",
        "ELEC_413_lukasc_BraggSet2Num14_1252.mat",
        "ELEC_413_lukasc_BraggSet2Num15_1253.mat",
        "ELEC_413_lukasc_BraggSet2Num16_1251.mat",
        "ELEC_413_lukasc_BraggSet2Num17_1258.mat",
        "ELEC_413_lukasc_BraggSet2Num18_1256.mat",
        "ELEC_413_lukasc_BraggSet2Num19_1257.mat",
        "ELEC_413_lukasc_BraggSet2Num1_1242.mat",
        "ELEC_413_lukasc_BraggSet2Num20_1255.mat",
        "ELEC_413_lukasc_BraggSet2Num21_1262.mat",
        "ELEC_413_lukasc_BraggSet2Num22_1260.mat",
        "ELEC_413_lukasc_BraggSet2Num23_1261.mat",
        "ELEC_413_lukasc_BraggSet2Num24_1259.mat",
        "ELEC_413_lukasc_BraggSet2Num2_1240.mat",
        "ELEC_413_lukasc_BraggSet2Num3_1241.mat",
        "ELEC_413_lukasc_BraggSet2Num4_1239.mat",
        "ELEC_413_lukasc_BraggSet2Num5_1246.mat",
        "ELEC_413_lukasc_BraggSet2Num6_1244.mat",
        "ELEC_413_lukasc_BraggSet2Num7_1245.mat",
        "ELEC_413_lukasc_BraggSet2Num8_1243.mat",
        "ELEC_413_lukasc_BraggSet2Num9_1250.mat",
        "ELEC_413_lukasc_BraggSet4Num10_1200.mat",
        "ELEC_413_lukasc_BraggSet4Num11_1201.mat",
        "ELEC_413_lukasc_BraggSet4Num12_1199.mat",
        "ELEC_413_lukasc_BraggSet4Num13_1206.mat",
        "ELEC_413_lukasc_BraggSet4Num14_1204.mat",
        "ELEC_413_lukasc_BraggSet4Num15_1205.mat",
        "ELEC_413_lukasc_BraggSet4Num16_1203.mat",
        "ELEC_413_lukasc_BraggSet4Num17_1210.mat",
        "ELEC_413_lukasc_BraggSet4Num18_1208.mat",
        "ELEC_413_lukasc_BraggSet4Num19_1209.mat",
        "ELEC_413_lukasc_BraggSet4Num1_1194.mat",
        "ELEC_413_lukasc_BraggSet4Num20_1207.mat",
        "ELEC_413_lukasc_BraggSet4Num21_1214.mat",
        "ELEC_413_lukasc_BraggSet4Num22_1212.mat",
        "ELEC_413_lukasc_BraggSet4Num23_1213.mat",
        "ELEC_413_lukasc_BraggSet4Num24_1211.mat",
        "ELEC_413_lukasc_BraggSet4Num2_1192.mat",
        "ELEC_413_lukasc_BraggSet4Num3_1193.mat",
        "ELEC_413_lukasc_BraggSet4Num4_1191.mat",
        "ELEC_413_lukasc_BraggSet4Num5_1198.mat",
        "ELEC_413_lukasc_BraggSet4Num6_1196.mat",
        "ELEC_413_lukasc_BraggSet4Num7_1197.mat",
        "ELEC_413_lukasc_BraggSet4Num8_1195.mat",
        "ELEC_413_lukasc_BraggSet4Num9_1202.mat",
    ]
}



import io
import os
import re

from setuptools import find_packages, setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding="utf-8") as fd:
        return re.sub(text_type(r":[a-z]+:`~?(.*?)`"), text_type(r"``\1``"), fd.read())


def get_install_requires():
    with open("requirements.txt", "r") as f:
        return [line.strip() for line in f.readlines() if not line.startswith("-")]


def get_install_requires_dev():
    with open("requirements_dev.txt", "r") as f:
        return [line.strip() for line in f.readlines() if not line.startswith("-")]


setup(
    name="ubcpdk",
    version="0.0.10",
    url="https://github.com/gdsfactory/ubc",
    license="MIT",
    author="gdsfactory",
    install_requires=get_install_requires(),
    description="UBC Siepic Ebeam PDK from edx course",
    long_description=read("README.md"),
    packages=find_packages(exclude=("tests",)),
    python_requires=">=3.7",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
    ],
    extras_require={
        "full": list(set(get_install_requires() + get_install_requires_dev())),
        "basic": get_install_requires(),
    },
)



import numpy as np
from ubc.da.read_mat import read_mat


def remove_baseline(wavelength: np.ndarray, power: np.ndarray, deg: int = 4):
    """
    Fit a polynomial ``p(x) = p[0] * x**deg + ... + p[deg]`` of degree `deg`
    returns power corrected without baseline
    """
    pfit = np.polyfit(wavelength - np.mean(wavelength), power, deg)
    power_baseline = np.polyval(pfit, wavelength - np.mean(wavelength))

    power_corrected = power - power_baseline
    power_corrected = power_corrected + max(power_baseline) - max(power)
    return power_corrected


if __name__ == "__main__":
    import matplotlib.pyplot as plt
    import ubc

    w, p = read_mat(ubc.PATH.mzi1)
    pc = remove_baseline(w, p)
    plt.plot(w, pc)
    plt.show()



"""
based on https://github.com/SiEPIC-Kits/SiEPIC_Photonics_Package
"""

import numpy as np
from ubcsp.waveguide import beta, neff, wavelength_um


def mzi_spectrum(
    L1_um,
    L2_um,
    wavelength_um=wavelength_um,
    beta=beta,
    alpha=1e-3,
    neff=neff,
    n1=2.4,
    n2=-1,
    n3=0,
):
    """Returns MZI spectrum:

    Args:
        L1_um
        L2_um
        wavelength_um
        beta: propagation constant
    """
    if callable(beta):
        beta = beta(wavelength_um, neff=neff, alpha=alpha, n1=n1, n2=n2, n3=n3)

    return 0.25 * np.abs(np.exp(-1j * beta * L1_um) + np.exp(-1j * beta * L2_um)) ** 2


if __name__ == "__main__":
    import matplotlib.pyplot as plt

    # plt.plot(wavelength_um, mzi_spectrum(100, 110))
    plt.plot(wavelength_um, 10 * np.log10(mzi_spectrum(L1_um=40, L2_um=255)))
    plt.show()



from gdsfactory.component import Component
from gdsfactory.components.ring_single_dut import ring_single_dut

from ubc.import_gds import import_gds


def crossing() -> Component:
    """TE waveguide crossing."""
    return import_gds("ebeam_crossing4", rename_ports=True)


def ring_with_crossing(**kwargs) -> Component:
    return ring_single_dut(component=crossing(), **kwargs)


if __name__ == "__main__":
    c = ring_with_crossing()
    c.show()
